@@ -3,6 +3,7 @@ function(obj, seg.Z, psi=stop("provide psi"), control = seg.control(), model = T
 #preliminary version that should account the bootstrap restart
 #o<-segmented.lm1(lm(y~x),seg.Z=~x,psi=.2,control=seg.control(display=FALSE,n.boot=20,it.max=5))
     n.Seg<-1
+    if(length(all.vars(seg.Z))>1 & !is.list(psi)) stop("`psi' should be a list with more than one covariate in `seg.Z'")
     if(is.list(psi)){
       if(length(all.vars(seg.Z))!=length(psi)) stop("A wrong number of terms in `seg.Z' or `psi'")
       if(any(is.na(match(all.vars(seg.Z),names(psi), nomatch = NA)))) stop("Variables in `seg.Z' and `psi' do not match")
@@ -57,19 +58,26 @@ function(obj, seg.Z, psi=stop("provide psi"), control = seg.control(), model = T
     mf$formula<-update.formula(mf$formula,paste(seg.Z,collapse=".+"))
 #    }
     mf <- eval(mf, parent.frame())
-    #id.offs<-pmatch("offset",names(mf)) #questa identifica il nome offset(..). ELiminarlo dal dataframe? non conviene
-    #       altrimenti nel model.frame non risulta l'offset
+    
     weights <- as.vector(model.weights(mf))
     offs <- as.vector(model.offset(mf))
 
     if(!is.null(Call$weights)){ #"(weights)"%in%names(mf)
-      names(mf)[which(names(mf)=="(weights)")]<-as.character(Call$weights)
+      names(mf)[which(names(mf)=="(weights)")]<- all.vars(Call$weights, functions=FALSE)#prima di 0.2.9-4 era as.character(Call$weights)
       # mf["(weights)"]<-weights
       }
     mt <- attr(mf, "terms")
     interc<-attr(mt,"intercept")
     y <- model.response(mf, "any")
+
     XREG <- if (!is.empty.model(mt)) model.matrix(mt, mf, contrasts)
+
+    #il cambio in mf da "offset(_nomevar_)" al "_nomevar_" deve avvenire dopo "model.matrix(mt, mf, contrasts)" 
+    if(!is.null(offs)){
+      id.offs<-pmatch("offset",names(mf)) #questa identifica il nome offset(..). ELiminarlo dal dataframe? non conviene altrimenti nel model.frame non risulta l'offset
+      names(mf)[id.offs]<- all.vars(formula(paste("~", names(mf)[id.offs])), functions=FALSE)
+      }
+
     namesXREG0<-colnames(XREG)
     #nameLeftSlopeZero<-setdiff(all.vars(seg.Z), all.vars(formula(obj)))
     nameLeftSlopeZero<-setdiff(all.vars(seg.Z), names(coef(obj))) #in questo modo riconosce che sin(x*pi) NON  x, ad esempio.
@@ -218,19 +226,30 @@ function(obj, seg.Z, psi=stop("provide psi"), control = seg.control(), model = T
         id.warn <- TRUE
     }
     Vxb <- V %*% diag(beta.c, ncol = length(beta.c))
-    colnames(U) <- colnames(Vxb) <-sapply(strsplit(nomiOK,"U"),function(x)x[2])
-    #colnames(U) <- paste(ripetizioni, nomiZ, sep = ".")
-    #colnames(Vxb) <- paste(ripetizioni, nomiZ, sep = ".")
-    nomiU <- paste("U", colnames(U), sep = "")
-    nomiVxb <- paste("psi", colnames(Vxb), sep = "")
-    for(i in 1:ncol(U)) {
-        mf[nomiU[i]]<-U[,i]
-        mf[nomiVxb[i]]<-Vxb[,i]
-        }
+
+    #se usi una procedura automatica devi cambiare ripetizioni, nomiU e nomiV, e quindi:
+    length.psi<-tapply(as.numeric(as.character(names(psi))), as.numeric(as.character(names(psi))), length)
+    forma.nomiU<-function(xx,yy)paste("U",1:xx, ".", yy, sep="")
+    forma.nomiVxb<-function(xx,yy)paste("psi",1:xx, ".", yy, sep="")
+    nomiU   <- unlist(mapply(forma.nomiU, length.psi, name.Z)) #invece di un ciclo #paste("U",1:length.psi[i], ".", name.Z[i])
+    nomiVxb <- unlist(mapply(forma.nomiVxb, length.psi, name.Z))
+    
+#    nomiVxb <-paste("psi",ripetizioni, ".",nomiZ ,sep="")
+#Dalla 0.2.9-5 eliminati i seguenti. La linea sopra sembra sufficiente
+#    colnames(U) <- colnames(Vxb) <-sapply(strsplit(nomiOK,"U"),function(x)x[2])
+#    #colnames(U) <- paste(ripetizioni, nomiZ, sep = ".")
+#    #colnames(Vxb) <- paste(ripetizioni, nomiZ, sep = ".")
+#    nomiU <- paste("U", colnames(U), sep = "")
+#    nomiVxb <- paste("psi", colnames(Vxb), sep = "")
 #    for (i in 1:ncol(U)) {
 #        assign(nomiU[i], U[, i], envir = KK)
 #        assign(nomiVxb[i], Vxb[, i], envir = KK)
 #    }
+
+    for(i in 1:ncol(U)) {
+        mf[nomiU[i]]<-U[,i]
+        mf[nomiVxb[i]]<-Vxb[,i]
+        }
     nnomi <- c(nomiU, nomiVxb)
     Fo <- update.formula(formula(obj0), as.formula(paste(".~.+",
         paste(nnomi, collapse = "+"))))
@@ -239,12 +258,18 @@ function(obj, seg.Z, psi=stop("provide psi"), control = seg.control(), model = T
     if(!is.null(objF[["subset"]])) objF[["subset"]]<-NULL
     objF<-eval(objF, envir=mf)
     if(!gap){
-        names.coef<-names(objF$coefficients)
-        objF$coefficients<-obj$coefficients
-        names(objF$coefficients)<-names.coef
-        objF$fitted.values<-obj$fitted.values
-        objF$residuals<-obj$residuals
-        }
+      names.coef<-names(objF$coefficients)
+      #questi codici funzionano e si basano sull'assunzioni che le U e le V siano ordinate..
+      if(k==1) {names(obj$coefficients)[match(c("U","V"), names(coef(obj)))]<- nnomi
+          } else {
+      names(obj$coefficients)[match(c(paste("U",1:k, sep=""), paste("V",1:k, sep="")), names(coef(obj)))]<- nnomi  
+          }
+      objF$coefficients[names.coef]<-obj$coefficients[names.coef] 
+        #objF$coefficients<-obj$coefficients
+        #names(objF$coefficients)<-names.coef
+      objF$fitted.values<-obj$fitted.values
+      objF$residuals<-obj$residuals
+      }
     if(any(is.na(objF$coefficients))){
     stop("some estimate is NA: premature stopping with a large number of breakpoints?",
       call. = FALSE)
